#include "AneuMeshLoader.h"
#include <fstream>
#include "Exceptions.h"

void AneuMeshLoader::loadMesh(const std::string& fileName) 
{
    size_t nodeNumber, dimension, elementNumber, nodesInElement, boundaryNumber, nodesInBoundary;
    std::ifstream inputFile(fileName);
    if (!inputFile)
        throw Exceptions();

    //   ()   
    inputFile >> nodeNumber >> dimension;
    nodeSTL.reserve(nodeNumber);
    for (int i = 0; i < nodeNumber; ++i) 
    {
        nodeSTL.emplace_back();
        nodeSTL[i].id = i + 1;
        inputFile >> nodeSTL[i].coordinates[0] >> nodeSTL[i].coordinates[1] >> nodeSTL[i].coordinates[2];
        nodeSTL[i].isVertex = false;
    }

    //    ()        ()
    inputFile >> elementNumber >> nodesInElement;
    finiteSTL.reserve(elementNumber);
    for (int i = 0; i < elementNumber; ++i) 
    {
        finiteSTL.emplace_back();
        finiteSTL[i].FiniteElementIdList.resize(nodesInElement);
        finiteSTL[i].id = i + 1;
        inputFile >> finiteSTL[i].MaterialId;
        for (auto& nodeId : finiteSTL[i].FiniteElementIdList)
            inputFile >> nodeId;
    }

    //     ,      ( 3-   - 3)
    inputFile >> boundaryNumber >> nodesInBoundary;
    boundarySTL.reserve(boundaryNumber);
    for (int i = 0; i < boundaryNumber; ++i) 
    {
        boundarySTL.emplace_back();
        boundarySTL[i].elementId = i + 1;
        inputFile >> boundarySTL[i].boundaryId;
        boundarySTL[i].BoundaryFiniteElementIdList.reserve(nodesInBoundary);
        for (auto& nodeId : boundarySTL[i].BoundaryFiniteElementIdList)
            inputFile >> nodeId;
    }
}