#include "MeshLoader.h"

const std::vector<Node>& MeshLoader::getNodeSTL() const 
{
    return nodeSTL;
}

const std::vector<FiniteElement>& MeshLoader::getFiniteSTL() const 
{
    return finiteSTL;
}

const std::vector<BoundaryFiniteElement>& MeshLoader::getBoundarySTL() const 
{
    return boundarySTL;
}

std::vector<size_t> MeshLoader::findFiniteElementById(int x, int y, int z) const 
{
    std::vector<size_t> element{};
    auto pred = [x, y, z](const FiniteElement& FE)
    {
        return std::find(FE.FiniteElementIdList.begin(), FE.FiniteElementIdList.end(), x) != FE.FiniteElementIdList.end()
            && std::find(FE.FiniteElementIdList.begin(), FE.FiniteElementIdList.end(), y) != FE.FiniteElementIdList.end()
            && std::find(FE.FiniteElementIdList.begin(), FE.FiniteElementIdList.end(), z) != FE.FiniteElementIdList.end();
    };
    auto iter = finiteSTL.begin();
    while (iter != finiteSTL.end()) 
    {
        iter = std::find_if(iter, finiteSTL.end(), pred);
        if (iter != finiteSTL.end()) 
        {
            element.push_back(iter->id);
            ++iter;
        }
    }
    return element;
}

std::vector<FiniteElement> MeshLoader::findFiniteElementByEdge(int x, int y) const 
{
    std::vector<FiniteElement> element{};
    auto pred = [x, y](const FiniteElement& FE)
    {
        return std::find(FE.FiniteElementIdList.begin(), FE.FiniteElementIdList.end(), x) != FE.FiniteElementIdList.end()
            && std::find(FE.FiniteElementIdList.begin(), FE.FiniteElementIdList.end(), y) != FE.FiniteElementIdList.end();
    };
    auto iter = finiteSTL.begin();
    while (iter != finiteSTL.end()) 
    {
        iter = std::find_if(iter, finiteSTL.end(), pred);
        if (iter != finiteSTL.end()) 
        {
            element.push_back(*iter);
            ++iter;
        }
    }
    return element;
}

std::vector<BoundaryFiniteElement> MeshLoader::getBoundaryFiniteElementSTLByBoundaryId(size_t boundaryId) const 
{
    std::vector<BoundaryFiniteElement> element{};
    auto pred = [boundaryId](const BoundaryFiniteElement& BFE) 
    {
        return BFE.boundaryId == boundaryId;
    };
    auto iter = boundarySTL.begin();
    while (iter != boundarySTL.end()) 
    {
        iter = std::find_if(iter, boundarySTL.end(), pred);
        if (iter != boundarySTL.end()) 
        {
            element.push_back(*iter);
            ++iter;
        }
    }
    return element;
}

std::set<size_t> MeshLoader::getNodeSTLByBoundaryId(size_t boundaryId) const 
{
    std::set<size_t> element{};
    auto pred = [boundaryId](const BoundaryFiniteElement& BFE) 
    {
        return BFE.boundaryId == boundaryId;
    };
    auto iter = boundarySTL.begin();
    while (iter != boundarySTL.end()) 
    {
        iter = std::find_if(iter, boundarySTL.end(), pred);
        if (iter != boundarySTL.end()) 
        {
            for (const auto& node : iter->BoundaryFiniteElementIdList) 
                element.insert(node);
            ++iter;
        }
    }
    return element;
}

std::vector<FiniteElement> MeshLoader::getFiniteElementByMaterial(size_t MaterialId) const 
{
    std::vector<FiniteElement> element{};
    auto pred = [MaterialId](const FiniteElement& FE) 
    {
        return MaterialId == FE.MaterialId;
    };
    auto iter = finiteSTL.begin();
    while (iter != finiteSTL.end()) 
    {
        iter = std::find_if(iter, finiteSTL.end(), pred);
        if (iter != finiteSTL.end()) 
        {
            element.push_back(*iter);
            ++iter;
        }
    }
    return element;
}

std::vector<std::set<size_t>> MeshLoader::getNeighbourSTL() const 
{
    std::vector<std::set<size_t>> vector(nodeSTL.size() + 1);
    for (const auto& element : finiteSTL)
        for (const auto& id : element.FiniteElementIdList)
            for (const auto& neighbourId : element.FiniteElementIdList)
                if (id != neighbourId)
                    vector[id].insert(neighbourId);
    return vector;
}

void MeshLoader::insertNodesInCenter(const FiniteElement& FE, size_t id1, size_t id2) 
{
    Node newNode{};
    newNode.id = nodeSTL.size();
    for (int i = 0; i < 2; ++i)
        newNode.coordinates[i] = (nodeSTL[id1].coordinates[i] + nodeSTL[id2].coordinates[i]) / 2;
    nodeSTL.push_back(newNode);
}
