#pragma once
#include <vector>
#include "Structures.h"

class MeshLoader 
{
protected:
    std::vector<Node> nodeSTL;
    std::vector<FiniteElement> finiteSTL;
    std::vector<BoundaryFiniteElement> boundarySTL;
public:
    virtual ~MeshLoader() = default;
    virtual void loadMesh(const std::string&) = 0;
    const std::vector<Node>& getNodeSTL() const;
    const std::vector<FiniteElement>& getFiniteSTL() const;
    const std::vector<BoundaryFiniteElement>& getBoundarySTL() const;
    std::vector<size_t> findFiniteElementById(int x, int y, int z) const;
    std::vector<FiniteElement> findFiniteElementByEdge(int x, int y) const;
    std::vector<FiniteElement> getFiniteElementByMaterial(size_t MaterialId) const;
    std::set<size_t> getNodeSTLByBoundaryId(size_t boundaryId) const;
    std::vector<BoundaryFiniteElement> getBoundaryFiniteElementSTLByBoundaryId(size_t boundaryId) const;
    std::vector<std::set<size_t>> getNeighbourSTL() const;
    void insertNodesInCenter(const FiniteElement&, size_t, size_t);
};
